#include "CopyKeys.h"

#include <maya/MSelectionList.h>
#include <maya/MGlobal.h>
#include <maya/MDistance.h>

#include <iostream>

#include "SpaceLocator.h"

VCopyKeys::VCopyKeys()
{
}

void * VCopyKeys::creator()
{ 	
	return new VCopyKeys(); 
}

MStatus VCopyKeys::doIt(const MArgList& args)
{
	return redoIt();
}

MStatus VCopyKeys::redoIt()
{
	MSelectionList SelList;
	MStringArray SelStrings;

	MGlobal::getActiveSelectionList(SelList);
	SelList.getSelectionStrings(SelStrings);

	int NumSel = SelStrings.length();

	// There should be a minimum of two objects selected - a 
	// source node and one (or more) target nodes

	if (NumSel >= 2)
	{
		std::vector< std::vector<std::string> > ChildLocators;
		ChildLocators.resize(NumSel);

		// The last selected node is the source hierarchy

		for (int i = 0; i < NumSel; i++)
		{
			std::string strSelection = SelStrings[i].asChar();

			FindAllChildLocators(strSelection, ChildLocators[i]);
		}

		CopySourceKeysToTargets(ChildLocators);

		// Restore original selection

		MGlobal::setActiveSelectionList(SelList);
	}

	return MS::kSuccess;
}

void VCopyKeys::FindAllChildLocators(std::string a_strTopNode, std::vector<std::string> & a_rChildLocators)
{
	if (IsLocatorName(a_strTopNode) )
	{
		a_rChildLocators.push_back(a_strTopNode);
	}
	else
	{
		// Do recursion for children

		std::string strGetChildren = "listRelatives -c -f ";	

		MStringArray ChildStrings;
		MGlobal::executeCommand((strGetChildren + a_strTopNode).c_str(), ChildStrings);

		int NumChildren = ChildStrings.length();

		for (int i = 0; i < NumChildren; i++)
		{
			std::string strChild = ChildStrings[i].asChar();

			FindAllChildLocators(strChild, a_rChildLocators);
		}
	}	
}

void VCopyKeys::CopySourceKeysToTargets(std::vector< std::vector<std::string> > & a_rChildLocators)
{
	// The last vector contains the source keys... all the others are targets

	int NumHierarchies = a_rChildLocators.size();
	std::vector<std::string> SourceKeys = *(a_rChildLocators.end() - 1);

	// Strip off the source vector from the end

	a_rChildLocators.erase(a_rChildLocators.end() - 1);

	std::vector<std::string>::iterator iSourceLocator;

	// Some commands we'll be needing... no point constantly making/deleting them

	std::string strCopyCmd = "copyKey ";
	std::string strPasteCmd = "pasteKey -o replace ";

	for (iSourceLocator = SourceKeys.begin(); iSourceLocator != SourceKeys.end(); iSourceLocator++)
	{
		// for each of these source locators... see if there is a matching name in any 
		// of the target hierarchies - if so, copy the keys across

		std::vector< std::vector<std::string> >::iterator iHierarchy;
		std::vector<std::string>::iterator iLocator;

		int SourceScopePos = (*iSourceLocator).rfind("|");
		std::string strSourceBody = (*iSourceLocator).substr(SourceScopePos);

		for (iHierarchy = a_rChildLocators.begin(); iHierarchy != a_rChildLocators.end(); iHierarchy++)
		{
			for (iLocator = (*iHierarchy).begin(); iLocator != (*iHierarchy).end(); iLocator++)
			{
				int DestScopePos = (*iLocator).rfind("|");
				std::string strDestBody = (*iLocator).substr(DestScopePos);

				if (strSourceBody == strDestBody)
				{
					// Found it... do the copy and then remove the locator from our target list
					// to avoid redundant searches...
#ifdef _DEBUG
					std::string strCopyTest = (strCopyCmd + (*iSourceLocator));
					std::string strPasteTest = (strPasteCmd + (*iLocator));
#endif

					MGlobal::executeCommand((strCopyCmd + (*iSourceLocator)).c_str());
					MGlobal::executeCommand((strPasteCmd + (*iLocator)).c_str());

					(*iHierarchy).erase(iLocator);
				}
			}
		}
	}
}

bool VCopyKeys::IsLocatorName(std::string a_strObjectName)
{
	bool bRetVal = false;

	std::string strGetChildren = "listRelatives -c -f ";	

	MStringArray ChildStrings;
	MGlobal::executeCommand((strGetChildren + a_strObjectName).c_str(), ChildStrings);

	int NumChildren = ChildStrings.length();

	for (int i = 0; i < NumChildren; i++)
	{
		std::string strChild = ChildStrings[i].asChar();

		if (strChild.find(DOF_LOCATOR) != -1 || strChild.find(MARKER_LOCATOR) != -1)
		{
			bRetVal = true;
			break;
		}
	}

	return bRetVal;
}

