///////////////////////////////////////////////////////////////////////////////
// FileContents.h
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <vector>

#include "FileSection.h"
#include "FileDynamicRecord.h"
#include <cassert>

class VFileContents 
{
private:
	SSHORT								m_Version;
	std::vector<VFileSection *>			m_Sections;
	std::vector<VFileDynamicRecord *>	m_DynamicRecs;

	int m_DynamicIdx;

public:
	VFileContents();
	virtual ~VFileContents();

	void SetVersion(SSHORT a_Vers) { m_Version = a_Vers; }

	bool AddSection(VFileSection * a_pSection);
	void AddDynamicRecord(VFileDynamicRecord * a_pDynamic);

	const VFileSection * GetSectionByName(std::string a_strTargetName) const;
	int GetNumSections() const { return m_Sections.size(); }

	// This is a temporary hack... (so is commented out f.t.m)

	const std::vector<VFileDynamicRecord *> GetDynamicRecords() { return m_DynamicRecs; }
//	int GetNumFrames() const { return m_DynamicRecs.size(); }
	int GetMinFrameNum() const;
	int GetMaxFrameNum() const;

	VFileSection * GetSectionAt(int a_Index);

	int GetNumDatagroups() const;
	bool GetLabelsForDatagroup(int a_GroupID, std::vector<std::string> & a_rLabels) const;
	float GetFrameRate(int a_GroupID) const;

	VFileDynamicRecord * GetDynamicRecord(int a_DatagroupID, int a_Frame);
	VFileDynamicRecord * GetFirstDynamic();
	VFileDynamicRecord * GetNextDynamic();
	VFileDynamicRecord * GetFirstDynamicOfGroup(int a_DatagroupID);
	VFileDynamicRecord * GetNextDynamicOfGroup(int a_DatagroupID);
};

