///////////////////////////////////////////////////////////////////////////////
// FileDatagroupSection.cpp
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#include "FileDatagroupSection.h"
#include <cassert>

VFileDatagroupSection::VFileDatagroupSection(int a_Len) : VFileSection(a_Len, DATAGROUP_SECTION)
{

}

VFileDatagroupSection::~VFileDatagroupSection()
{
	std::vector<VFileDatagroupRecord *>::iterator RecordIt;

	for (RecordIt = m_pRecords.begin(); RecordIt != m_pRecords.end(); RecordIt++)
	{
		delete *RecordIt;
		*RecordIt = NULL;
	}
}

VFileDatagroupRecord * VFileDatagroupSection::FindDatagroupByID(SSHORT a_TargID)
{
	SSHORT GroupID = -1;
	std::vector<VFileDatagroupRecord *>::iterator RecordIt;

	for (RecordIt = m_pRecords.begin(); RecordIt != m_pRecords.end(); RecordIt++)
	{
		if ((*RecordIt)->GetGroupID() == a_TargID)
		{
			return *RecordIt;
		}
	}

	return NULL;
}

const VFileDatagroupRecord * VFileDatagroupSection::FindDatagroupByID(SSHORT a_TargID) const
{
	SSHORT GroupID = -1;
	std::vector<VFileDatagroupRecord *>::const_iterator RecordIt;

//	for (RecordIt = const_cast<std::vector<VFileDatagroupRecord *>::iterator>(m_pRecords.begin()); RecordIt != m_pRecords.end(); RecordIt++)
	for (RecordIt = m_pRecords.begin(); RecordIt != m_pRecords.end(); RecordIt++)	
	{
		if ((*RecordIt)->GetGroupID() == a_TargID)
		{
			return *RecordIt;
		}
	}

	return NULL;
}




SLONG VFileDatagroupSection::GetDataLength()
{
	SLONG DataLen = 0;
	std::vector<VFileDatagroupRecord *>::iterator RecordIt;

	for (RecordIt = m_pRecords.begin(); RecordIt != m_pRecords.end(); RecordIt++)
	{
		DataLen += sizeof(SSHORT);				// Don't forget the header
		DataLen += (*RecordIt)->GetLength();
	}

	DataLen += sizeof(SSHORT);	// Section terminator	

	return DataLen;
}

const VFileDatagroupRecord * VFileDatagroupSection::GetRecordAt(int a_Index) const
{
	assert(a_Index >= 0);

	return m_pRecords[a_Index];
}

VFileDatagroupRecord * VFileDatagroupSection::GetRecordAt(int a_Index)
{
	assert(a_Index >= 0);

	return m_pRecords[a_Index];
}

void VFileDatagroupSection::PrepareForWrite()
{
	std::vector<VFileDatagroupRecord *>::iterator RecordIt;

	for (RecordIt = m_pRecords.begin(); RecordIt != m_pRecords.end(); RecordIt++)
	{
		(*RecordIt)->BuildBytesFromRecord();
	}
}

