#include "CreateOffsetCubes.h"
#include "SpaceLocator.h"

#include <maya/MSelectionList.h>
#include <maya/MGlobal.h>
#include <maya/MDistance.h>

#include <iostream>

VCreateOffsetCubes::VCreateOffsetCubes()
{
	MDistance CubeDim(CUBE_DIM_IN_CM, MDistance::kCentimeters);
	m_CubeDim = CubeDim.as(MDistance::uiUnit());
}

void * VCreateOffsetCubes::creator()
{ 	
	return new VCreateOffsetCubes(); 
}

bool VCreateOffsetCubes::hasSyntax() const
{
	return true;
}

bool VCreateOffsetCubes::isUndoable() const
{
	return false;
}

MStatus VCreateOffsetCubes::doIt(const MArgList& args)
{
	return redoIt();
}

MStatus VCreateOffsetCubes::redoIt()
{
	MSelectionList SelList;
	MStringArray SelStrings;

	// Create cubes for all selected nodes and their children

	MGlobal::getActiveSelectionList(SelList);
	SelList.getSelectionStrings(SelStrings);

	int NumSel = SelStrings.length();

	for (int i = 0; i < NumSel; i++)
	{
		std::string strSelection = SelStrings[i].asChar();

		CreateCubesForChildren(strSelection);
	}

	// Restore original selection

	MGlobal::setActiveSelectionList(SelList);

	return MS::kSuccess;
}

void VCreateOffsetCubes::CreateCubesForChildren(std::string a_strObjectName)
{
	if (IsDofLocatorName(a_strObjectName))
	{
		// Create the cubes for this node...

		std::string strCubeSuffix = a_strObjectName;
		int PipePos = strCubeSuffix.rfind("|");

		if (PipePos != -1)
			strCubeSuffix = strCubeSuffix.substr(PipePos + 1);

		CreateCube("Rot_" + strCubeSuffix, a_strObjectName);
		CreateCube("Pos_" + strCubeSuffix, a_strObjectName);

#ifdef _DEBUG
		std::cerr << a_strObjectName.c_str() << std::endl;
#endif
	}
	else
	{
		// Do recursion for children

		std::string strGetChildren = "listRelatives -c -f ";	

		MStringArray ChildStrings;
		MGlobal::executeCommand((strGetChildren + a_strObjectName).c_str(), ChildStrings);

		int NumChildren = ChildStrings.length();

		for (int i = 0; i < NumChildren; i++)
		{
			std::string strChild = ChildStrings[i].asChar();

			CreateCubesForChildren(strChild);
		}
	}
}

bool VCreateOffsetCubes::IsDofLocatorName(std::string a_strObjectName)
{
	bool bRetVal = false;

	std::string strGetChildren = "listRelatives -c -f ";	

	MStringArray ChildStrings;
	MGlobal::executeCommand((strGetChildren + a_strObjectName).c_str(), ChildStrings);

	int NumChildren = ChildStrings.length();

	for (int i = 0; i < NumChildren; i++)
	{
		std::string strChild = ChildStrings[i].asChar();

		if (strChild.find(DOF_LOCATOR) != -1)
		{
			bRetVal = true;
			break;
		}
	}

	return bRetVal;
}

void VCreateOffsetCubes::CreateCube(std::string a_strCubeName, std::string a_strLocatorName)
{
	char CubeBuf[100] = {0};

	sprintf(CubeBuf, "polyCube -w %f -h %f -d %f -sx 1 -sy 1 -sz 1 -ax 0 1 0 -tx 1 -ch 0 -n ", m_CubeDim, m_CubeDim, m_CubeDim);
	std::string strCreateCubeCmd = CubeBuf;

	// First create the cube

	MGlobal::executeCommand((strCreateCubeCmd + a_strCubeName).c_str());

	// now select the cube.. and then the locator

	MGlobal::executeCommand(("select -r " + a_strCubeName).c_str());
	MGlobal::executeCommand(("select -tgl " + a_strLocatorName).c_str());

	// ...and parent

	MGlobal::executeCommand("parent");

	// select the locator first... then the cube

	MGlobal::executeCommand(("select -r " + a_strLocatorName).c_str());
	MGlobal::executeCommand(("select -tgl " + a_strCubeName).c_str());

	// point constrain to position the cube correctly.. 

	MGlobal::executeCommand("pointConstraint");
	MGlobal::executeCommand("orientConstraint");

	// If we opt to delete the constraints as soon as the offsetcubes are 
	// correctly positioned then the cubes return to the origin!! Don't know
	// why this happens... but it only happens in 2.5.

#ifdef MAYA3
	//...then unconstrain by selecting and deleting the constraint

	MGlobal::executeCommand(("select -r " + a_strCubeName).c_str());
	MGlobal::executeCommand("delete -cn");
#endif
}

